<?php
/**
 * PHPTAL templating engine
 *
 * PHP Version 5
 *
 * @category HTML
 * @package  PHPTAL
 * @author   Laurent Bedubourg <lbedubourg@motion-twin.com>
 * @author   Kornel Lesiński <kornel@aardvarkmedia.co.uk>
 * @license  http://www.gnu.org/licenses/lgpl.html GNU Lesser General Public License
 * @version  SVN: $Id: UseMacro.php 500 2009-03-31 13:05:37Z kornel $
 * @link     http://phptal.motion-twin.com/ 
 */
// METAL Specification 1.0
//
//      argument ::= expression
//
// Example:
// 
//      <hr />
//      <p metal:use-macro="here/master_page/macros/copyright">
//      <hr />
//
// PHPTAL: (here not supported)
//
//      <?php echo phptal_macro( $tpl, 'master_page.html/macros/copyright'); ? >
//

/**
 * @package PHPTAL.php.attribute.metal
 * @author Laurent Bedubourg <lbedubourg@motion-twin.com>
 */
class PHPTAL_Php_Attribute_METAL_UseMacro extends PHPTAL_Php_Attribute
{
    static $ALLOWED_ATTRIBUTES = array(
        'fill-slot'=>'http://xml.zope.org/namespaces/metal', 
        'define-macro'=>'http://xml.zope.org/namespaces/metal', 
        'define'=>'http://xml.zope.org/namespaces/tal',
    );
    
    public function start(PHPTAL_Php_CodeWriter $codewriter)
    {
        $this->pushSlots($codewriter);
        
        foreach ($this->phpelement->childNodes as $child) {
            $this->generateFillSlots($codewriter, $child);
        }

        $macroname = strtr($this->expression,'-','_');

        // local macro (no filename specified) and non dynamic macro name
        // can be called directly if it's a known function (just generated or seen in previous compilation)
        if (preg_match('/^[a-z0-9_]+$/i', $macroname) && $codewriter->functionExists($macroname)) {
            $code = $codewriter->getFunctionPrefix() . $macroname . '($_thistpl, $tpl)';
            $codewriter->pushCode($code);
        }
        // external macro or ${macroname}, use PHPTAL at runtime to resolve it
        else {
            $code = $codewriter->interpolateTalesVarsInString($this->expression);
            $codewriter->pushCode('$tpl->_executeMacroOfTempalte('.$code.', $_thistpl)');
        }

        $this->popSlots($codewriter);
    }
    
    public function end(PHPTAL_Php_CodeWriter $codewriter)
    {
    }

    private function pushSlots(PHPTAL_Php_CodeWriter $codewriter)
    {
        // reset template slots on each macro call ?
        // 
        // NOTE: defining a macro and using another macro on the same tag 
        // means inheriting from the used macro, thus slots are shared, it 
        // is a little tricky to understand but very natural to use.
        //
        // For example, we may have a main design.html containing our main 
        // website presentation with some slots (menu, content, etc...) then
        // we may define a member.html macro which use the design.html macro
        // for the general layout, fill the menu slot and let caller templates
        // fill the parent content slot without interfering. 
        if (!$this->phpelement->hasAttributeNS('http://xml.zope.org/namespaces/metal','define-macro')) {
            $codewriter->pushCode('$ctx->pushSlots()');
        }
    }

    private function popSlots(PHPTAL_Php_CodeWriter $codewriter)
    {
        // restore slots if not inherited macro
        if (!$this->phpelement->hasAttributeNS('http://xml.zope.org/namespaces/metal','define-macro')) {
            $codewriter->pushCode('$ctx->popSlots()');
        }
    }
    
    private function generateFillSlots(PHPTAL_Php_CodeWriter $codewriter, PHPTAL_DOMNode $phpelement)
    {
        if (false == ($phpelement instanceOf PHPTAL_DOMElement)) 
            return;

        // if the tag contains one of the allowed attribute, we generate it
        foreach (self::$ALLOWED_ATTRIBUTES as $qname => $uri) {
            if ($phpelement->hasAttributeNS($uri, $qname)) {
                $phpelement->generate($codewriter);
                return;
            }
        }
        
        // recurse
        foreach ($phpelement->childNodes as $child) {
            $this->generateFillSlots($codewriter, $child);
        }
    }
}

?>
